import React from "/apps/websight-atlaskit-esm/web-resources/react.js";
import styled from "/apps/websight-atlaskit-esm/web-resources/styled-components.js";
import { colors } from "/apps/websight-admin/web-resources/theme.js";
import { DateTimeOffset } from "/apps/websight-admin/web-resources/DateUtils.js";
import InfoIconPopup from "./InfoIconPopup.js";
const DependenciesList = styled.ul`
    padding-left: 20px;
    margin-top: 6px;
`;
const Dependency = styled.li`
    font-size: 12px;
`;
const UnresolvedDependency = styled.li`
    color: ${colors.red};
    font-size: 12px;
`;
const Description = styled.span`
    font-size: 12px;
    word-wrap: break-word;
`;

const PackageDefinitionInfo = props => {
  const {
    packageData,
    onClick
  } = props;
  let dependencies;
  let description;
  let hasUnresolvedDependencies = false;

  if (packageData.dependencies && packageData.dependencies.length > 0) {
    dependencies = /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("span", null, "Dependencies"), /*#__PURE__*/React.createElement(DependenciesList, null, packageData.dependencies.map((dependency, index) => dependency.resolved ? /*#__PURE__*/React.createElement(Dependency, {
      key: index
    }, dependency.dependency) : /*#__PURE__*/React.createElement(UnresolvedDependency, {
      key: index
    }, dependency.dependency, " (unresolved)"))));
    hasUnresolvedDependencies = packageData.dependencies.find(dependency => !dependency.resolved);
  }

  if (packageData.description) {
    description = /*#__PURE__*/React.createElement("div", null, dependencies && /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("br", null), /*#__PURE__*/React.createElement("span", null, "Description"), /*#__PURE__*/React.createElement("br", null)), /*#__PURE__*/React.createElement(Description, null, packageData.description));
  }

  if (dependencies || description) {
    return /*#__PURE__*/React.createElement(InfoIconPopup, {
      content: /*#__PURE__*/React.createElement(React.Fragment, null, dependencies, description),
      icon: "info_outline",
      iconStyle: {
        color: hasUnresolvedDependencies ? colors.red : undefined
      },
      onClick: onClick
    });
  } else {
    return null;
  }
};

const PackageScheduleInfo = props => {
  const {
    packageData,
    onClick
  } = props;

  if (packageData.nextScheduledAction) {
    const nextScheduledAction = packageData.nextScheduledAction;
    const actionName = nextScheduledAction.action && nextScheduledAction.action.toLowerCase();
    const executionDateTimeOffset = new DateTimeOffset(nextScheduledAction.nextExecution);
    const executionTime = executionDateTimeOffset.getEnGbDateTimeOffsetString();
    return /*#__PURE__*/React.createElement(InfoIconPopup, {
      content: /*#__PURE__*/React.createElement(Description, null, "Next ", actionName, " scheduled at ", executionTime, " by ", nextScheduledAction.applicantId),
      icon: "schedule",
      onClick: onClick
    });
  }

  return null;
};

const PackageInfo = props => {
  return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(PackageDefinitionInfo, props), /*#__PURE__*/React.createElement(PackageScheduleInfo, props));
};

export default PackageInfo;