function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import React from "/apps/websight-atlaskit-esm/web-resources/react.js";
import Badge from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/badge.js";
import Popup from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/popup.js";
import Tooltip from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/tooltip.js";
import styled from "/apps/websight-atlaskit-esm/web-resources/styled-components.js";
import { colors } from "/apps/websight-admin/web-resources/theme.js";
import Accordion from "./Accordion.js";
import ChangelogReadService from "../services/ChangelogReadService.js";
import { PROPERTY_TYPE, SAVE_OPERATIONS } from "../utils/ResourceBrowserConstants.js";
const PopupContainer = styled.div`
    align-items: center
    display: flex;
    height: 100%;
    margin-right: 4px;
`;
const InfoIcon = styled.i`
    color: ${colors.lightGrey};
    cursor: pointer;
    display: flex;
    font-size: 26px;
`;
const ContentWrapper = styled.div`
    font-size: 12px;
    max-height: 75vh;
    max-height: 75vh;
    max-width: 450px;
    min-width: 450px;
    overflow-y: auto;
    overflow-y: auto;
    padding: 30px;
`;
const FieldContainer = styled.div`
    display: flex;
    align-items: center;
    .name {
        display: inline-block;
        width: 30%;
    }
    .value {
        display: inline-block;
        width: 70%;
    }
`;
const Title = styled.h4`
    -webkit-box-align: center;
    align-items: center;
    display: flex;
    font-size: 20px;
    font-style: inherit;
    font-weight: 500;
    letter-spacing: -0.008em;
    line-height: 1;
    margin: 0px;
    min-width: 0px;
    margin-bottom: 22px;
`;
const ChangeAmountWrapper = styled.div`
    position: absolute;
    top: -7px;
    right: -3px;
    cursor: pointer;
    span {
        font-size: 12px;
    }
`;
const InlineBadgeWrapper = styled.div`
    display: inline-block;
    margin: 0 5px;
    span {
        font-size: 10px;
        font-weight: bold;
    }
`;
const AccordionTitleContainer = styled.div`
    display: flex;
    width: 100%;
    justify-content: space-between;
`;
export default class ChangelogPopup extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      isOpen: false
    };
    this.open = this.open.bind(this);
    this.close = this.close.bind(this);
  }

  open() {
    this.setState({
      isOpen: true
    });
  }

  close() {
    this.setState({
      isOpen: false
    });
  }

  changelogRender(changelog) {
    const isEmpty = ChangelogReadService.isEmpty(changelog);
    return /*#__PURE__*/React.createElement(React.Fragment, null, isEmpty ? /*#__PURE__*/React.createElement("span", null, "No changes to save.") : Object.keys(changelog).map(path => this.changelogPathRender(changelog[path], path)));
  }

  changelogPathRender(changelog, path) {
    const changeAmount = this.countChangesPath(changelog);
    return /*#__PURE__*/React.createElement(React.Fragment, null, changeAmount > 0 && /*#__PURE__*/React.createElement(Accordion, {
      title: /*#__PURE__*/React.createElement(AccordionTitleContainer, null, /*#__PURE__*/React.createElement("strong", null, path), /*#__PURE__*/React.createElement(InlineBadgeWrapper, null, /*#__PURE__*/React.createElement(Badge, {
        className: "inline-bade"
      }, changeAmount)))
    }, changelog.map(variantChangelog => this.changelogVariantRender(variantChangelog))));
  }

  changelogVariantRender(changelog) {
    const changeAmount = this.countChangesVariant(changelog);
    return /*#__PURE__*/React.createElement(React.Fragment, null, changeAmount > 0 && /*#__PURE__*/React.createElement(Accordion, {
      title: /*#__PURE__*/React.createElement(AccordionTitleContainer, null, /*#__PURE__*/React.createElement("span", null, changelog.provider), /*#__PURE__*/React.createElement(InlineBadgeWrapper, null, /*#__PURE__*/React.createElement(Badge, {
        className: "inline-bade"
      }, changeAmount)))
    }, Object.values(SAVE_OPERATIONS).filter(operation => changelog[operation]).map(operation => this.changelogVariantOperationRender(changelog[operation], operation))));
  }

  changelogVariantOperationRender(operation, name) {
    const operationRender = operationToRender => {
      return /*#__PURE__*/React.createElement(React.Fragment, null, Array.isArray(operationToRender) ? operationToRender.map((entry, index) => operationEntryRender(entry, index)) : operationEntryRender(operationToRender));
    };

    const operationEntryRender = (entry, title) => {
      if (typeof entry === 'object') {
        const fields = /*#__PURE__*/React.createElement(React.Fragment, null, Object.keys(entry).filter(fieldName => entry.type !== PROPERTY_TYPE.BINARY || fieldName !== 'value').map(fieldName => operationFieldRender(fieldName, operationRender(entry[fieldName]))));
        return /*#__PURE__*/React.createElement(React.Fragment, null, title !== undefined ? /*#__PURE__*/React.createElement(Accordion, {
          title: /*#__PURE__*/React.createElement("span", null, title)
        }, fields) : fields);
      }

      return /*#__PURE__*/React.createElement("span", null, operationFieldRender(entry));
    };

    const operationFieldRender = (fieldName, fieldValue) => {
      return /*#__PURE__*/React.createElement(FieldContainer, {
        key: fieldName
      }, /*#__PURE__*/React.createElement("span", {
        className: "name"
      }, fieldName), /*#__PURE__*/React.createElement("span", {
        className: "value"
      }, fieldValue));
    };

    const changeAmount = this.countChangesOperation(operation);
    return /*#__PURE__*/React.createElement(React.Fragment, null, changeAmount > 0 && /*#__PURE__*/React.createElement(Accordion, {
      title: /*#__PURE__*/React.createElement(AccordionTitleContainer, null, /*#__PURE__*/React.createElement("span", null, name), /*#__PURE__*/React.createElement(InlineBadgeWrapper, null, /*#__PURE__*/React.createElement(Badge, {
        className: "inline-bade"
      }, changeAmount)))
    }, operationRender(operation)));
  }

  countChangesOperation(changelog) {
    let result = 0;

    if (Array.isArray(changelog)) {
      changelog.forEach(entry => {
        result++;

        if (entry.children && Array.isArray(entry.children)) {
          result += this.countChangesOperation(entry.children);
        }
      });
    } else {
      result = 1;
    }

    return result;
  }

  countChangesVariant(changelog) {
    let result = 0;
    Object.values(SAVE_OPERATIONS).filter(operationName => changelog[operationName]).map(operationName => changelog[operationName]).forEach(operation => {
      result += this.countChangesOperation(operation);
    });
    return result;
  }

  countChangesPath(changelog) {
    let result = 0;
    changelog.forEach(variant => {
      result += this.countChangesVariant(variant);
    });
    return result;
  }

  countChangesChangelog(changelog) {
    let result = 0;
    Object.keys(changelog).forEach(path => {
      result += this.countChangesPath(changelog[path]);
    });
    return result;
  }

  render() {
    const {
      isOpen
    } = this.state;
    const {
      changelog
    } = this.props;
    const changeAmount = this.countChangesChangelog(changelog);
    return /*#__PURE__*/React.createElement(PopupContainer, null, /*#__PURE__*/React.createElement(Popup, {
      isOpen: isOpen,
      onClose: this.close,
      shouldFlip: true,
      content: () => /*#__PURE__*/React.createElement(ContentWrapper, null, /*#__PURE__*/React.createElement(Title, null, "Local Changes"), this.changelogRender(changelog)),
      trigger: triggerProps => /*#__PURE__*/React.createElement("div", _extends({}, triggerProps, {
        style: {
          position: 'relative'
        },
        onClick: this.open
      }), changeAmount > 0 && /*#__PURE__*/React.createElement(ChangeAmountWrapper, null, /*#__PURE__*/React.createElement(Badge, {
        appearance: "important"
      }, changeAmount)), /*#__PURE__*/React.createElement(Tooltip, {
        content: "Local Changes",
        delay: 0
      }, /*#__PURE__*/React.createElement(InfoIcon, {
        className: "material-icons"
      }, "list_alt")))
    }));
  }

}