import React from "/apps/websight-atlaskit-esm/web-resources/react.js";
import Button from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/button.js";
import { Cell, Row, Rows } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/table-tree.js";
import InlineEdit from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/inline-edit.js";
import Select from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/select.js";
import TextField from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/textfield.js";
import Tooltip from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/tooltip.js";
import styled from "/apps/websight-atlaskit-esm/web-resources/styled-components.js";
import { colors } from "/apps/websight-admin/web-resources/theme.js";
import InlineEditContainer from "./PropertyInlineEditContainer.js";
import PropertyMultiField from "./PropertyMultiField.js";
import PropertyValueField from "./PropertyValueField.js";
import { ARRAY_SUFFIX, PROPERTY_TYPE } from "../../utils/ResourceBrowserConstants.js";
const Actionbar = styled.div`
    padding: 6px 12px;
`;
const CreatedPropertyWrapper = styled.div`
    border-left: 2px solid ${colors.green};
    padding-right: 20px;
    font-weight: bold;
    input {
        font-weight: bold;
    }
    &:nth-of-type(even),
    &:nth-of-type(even) input + div > i,
    &:nth-of-type(even) input {
        background: ${colors.veryLightGrey}
    }
`;
const Icon = styled.i`
    color: ${colors.grey};
    cursor: pointer;
    display: flex;
    font-size: 12px;
    margin: 0 2px;
`;
const DuplicatedIcon = styled.i`
    cursor: pointer;
    color: ${colors.red};
    padding-left: 5px;
    font-size: 12px;
    line-height: 30px;
`;
const cellStyle = {
  alignItems: 'flex-start'
};
const deleteCellStyle = {
  paddingTop: '5px',
  alignItems: 'flex-start'
};
const operationButtonStyle = {
  height: '24px',
  position: 'absolute',
  right: '10px',
  top: '3px',
  width: '26px'
};
export default class PropertyCreationSection extends React.Component {
  constructor(props) {
    super(props);
    this.createPropertyRow = this.createPropertyRow.bind(this);
  }

  createPropertyRow(property) {
    const {
      name,
      type
    } = property;
    const {
      resource,
      changelog,
      onChangelogUpdate,
      onCreatedPropertyChange,
      onCreatedPropertyRemove,
      isDuplicated,
      provider
    } = this.props;
    const isMultivalued = type.endsWith(ARRAY_SUFFIX);

    const onFieldChange = param => {
      onCreatedPropertyChange(param);
    };

    const createTypeOptions = () => {
      return Object.values(PROPERTY_TYPE).reduce((result, propertyType) => {
        result.push({
          label: propertyType,
          value: propertyType
        });
        result.push({
          label: propertyType + ARRAY_SUFFIX,
          value: propertyType + ARRAY_SUFFIX
        });
        return result;
      }, []);
    };

    const onRevertValue = id => {
      onCreatedPropertyChange({ ...property,
        value: (property.value || []).filter((pValue, index) => index !== id)
      });
    };

    return /*#__PURE__*/React.createElement(CreatedPropertyWrapper, null, /*#__PURE__*/React.createElement(Row, {
      key: name,
      itemId: name
    }, /*#__PURE__*/React.createElement(Cell, {
      singleLine: true,
      style: cellStyle
    }, /*#__PURE__*/React.createElement(InlineEditContainer, {
      className: "new-property-name"
    }, /*#__PURE__*/React.createElement(TextField, {
      defaultValue: name,
      placeholder: "Name",
      className: "inline-editfield",
      elemAfterInput: isDuplicated(property) && /*#__PURE__*/React.createElement(Tooltip, {
        content: "Property will not be saved because of duplicated name",
        delay: 0
      }, /*#__PURE__*/React.createElement(DuplicatedIcon, {
        className: "material-icons"
      }, "warning")),
      appearance: "subtle",
      isCompact: true,
      autoFocus: true,
      onChange: event => onFieldChange({ ...property,
        name: event.target.value
      })
    }))), /*#__PURE__*/React.createElement(Cell, {
      style: cellStyle
    }, /*#__PURE__*/React.createElement(InlineEditContainer, {
      className: "new-property-type"
    }, /*#__PURE__*/React.createElement(InlineEdit, {
      readViewFitContainerWidth: true,
      hideActionButtons: true,
      editView: () => {
        return /*#__PURE__*/React.createElement(Select, {
          className: "inline-editfield",
          defaultValue: {
            label: PROPERTY_TYPE.STRING,
            value: PROPERTY_TYPE.STRING
          },
          value: type ? {
            label: type,
            value: type
          } : undefined,
          spacing: "compact",
          options: createTypeOptions(),
          closeMenuOnSelect: true,
          blurInputOnSelect: true,
          defaultMenuIsOpen: true,
          autoFocus: true,
          menuPosition: "fixed",
          menuPlacement: "top",
          onChange: option => {
            onFieldChange({ ...property,
              type: option.value,
              value: undefined
            });
          }
        });
      },
      readView: () => /*#__PURE__*/React.createElement("span", null, type),
      onConfirm: () => {// handled on edit view level
      }
    }))), /*#__PURE__*/React.createElement(Cell, {
      style: cellStyle
    }, isMultivalued ? /*#__PURE__*/React.createElement(PropertyMultiField, {
      changelog: changelog,
      onChange: (pName, pType, pValue) => onFieldChange({ ...property,
        value: pValue
      }),
      onChangelogUpdate: onChangelogUpdate,
      onUndo: onRevertValue,
      onRemove: onRevertValue,
      property: property,
      provider: provider,
      resource: resource,
      isNewProperty: true
    }) : /*#__PURE__*/React.createElement(PropertyValueField, {
      changelog: changelog,
      hideActions: true,
      onChange: (pName, pType, pValue) => onFieldChange({ ...property,
        value: pValue
      }),
      onChangelogUpdate: onChangelogUpdate,
      property: property,
      provider: provider,
      resource: resource
    })), /*#__PURE__*/React.createElement(Cell, null), /*#__PURE__*/React.createElement(Cell, {
      style: deleteCellStyle
    }, /*#__PURE__*/React.createElement(Tooltip, {
      content: "Remove",
      position: "top",
      delay: 0
    }, /*#__PURE__*/React.createElement(Button, {
      label: "Delete Resource",
      spacing: "compact",
      onClick: () => onCreatedPropertyRemove(property),
      style: operationButtonStyle,
      iconBefore: /*#__PURE__*/React.createElement(Icon, {
        className: "material-icons"
      }, "undo")
    })))));
  }

  render() {
    const {
      onCreateProperty,
      createdProperties
    } = this.props;
    return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(Rows, {
      items: createdProperties,
      render: property => this.createPropertyRow(property)
    }), /*#__PURE__*/React.createElement(Actionbar, null, /*#__PURE__*/React.createElement(Button, {
      onClick: () => onCreateProperty()
    }, "Add Property")));
  }

}