function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import React from "/apps/websight-atlaskit-esm/web-resources/react.js";
import Button, { ButtonGroup } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/button.js";
import { DateTimePicker } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/datetime-picker.js";
import ModalDialog, { ModalHeader, ModalTitle, ModalTransition } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/modal-dialog.js";
import Select from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/select.js";
import TextField from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/textfield.js";
import SectionMessage from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/section-message.js";
import FilePicker from "/apps/websight-admin/web-resources/FilePicker.js";
import Form, { FormFooter } from "/apps/websight-rest-atlaskit-client/web-resources/js/Form.js";
import ChangelogWriteService from "../../services/ChangelogWriteService.js";
import ResourceService from "../../services/ResourceService.js";
import { getResourcePath } from "../../utils/ChangelogUtil.js";
import { PROPERTY_TYPE } from "../../utils/ResourceBrowserConstants.js";
import { extractParentWithChild } from "../../utils/ResourceBrowserUtil.js";

const getMandatoryPropertyField = property => {
  const {
    name,
    type
  } = property;
  const defaultProps = {
    key: name,
    label: name,
    name: name,
    isRequired: true
  };

  if (type === PROPERTY_TYPE.BINARY) {
    return /*#__PURE__*/React.createElement(FilePicker, _extends({}, defaultProps, {
      placeholder: "Browse File"
    }));
  } else if (type === PROPERTY_TYPE.DATE) {
    return /*#__PURE__*/React.createElement(DateTimePicker, defaultProps);
  } else if (type === PROPERTY_TYPE.BOOLEAN) {
    return /*#__PURE__*/React.createElement(Select, _extends({}, defaultProps, {
      defaultValue: {
        label: 'true',
        value: true
      },
      options: [{
        label: 'true',
        value: true
      }, {
        label: 'false',
        value: false
      }]
    }));
  } else {
    return /*#__PURE__*/React.createElement(TextField, defaultProps);
  }
};

export default class CreateResourceModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      newNodeName: '',
      availableTypes: {},
      defaultTypes: {},
      selectedType: '',
      selectedProvider: '',
      isOpen: false,
      resource: {}
    };
    this.close = this.close.bind(this);
    this.open = this.open.bind(this);
    this.onSubmit = this.onSubmit.bind(this);
    this.validateName = this.validateName.bind(this);
    this.nodeTypeSelected = this.nodeTypeSelected.bind(this);
  }

  close() {
    this.setState({
      isOpen: false
    });
  }

  open(resource) {
    const defaultProvider = (((resource || {}).providers || {})[0] || {}).value;
    ResourceService.getResourceCreationInfo({
      parentPath: resource.path,
      parentType: resource.type
    }, info => {
      const availableTypes = {};
      const defaultTypes = {};
      Object.keys(info.detailedInfo).forEach(providerName => {
        availableTypes[providerName] = info.detailedInfo[providerName].allowedChildrenTypes;
        defaultTypes[providerName] = info.detailedInfo[providerName].defaultChildrenType;
      });
      this.setState({
        availableTypes: availableTypes,
        defaultTypes: defaultTypes,
        selectedProvider: defaultProvider,
        selectedType: defaultTypes[defaultProvider] || '',
        isOpen: true,
        resource: resource
      });
      this.nodeTypeSelected(this.state.selectedType, defaultProvider, availableTypes);
    });
  }

  mapMandatoryProperties(properties, formData) {
    const mappedProperties = [];

    const pushMandatoryProperty = (name, type, value) => {
      mappedProperties.push({
        name: name,
        type: type,
        value: value
      });
    };

    properties.forEach(mandatoryProperty => {
      if (mandatoryProperty.type === PROPERTY_TYPE.BINARY) {
        ChangelogWriteService.convertBinaryValue(formData[mandatoryProperty.name]).then(result => {
          pushMandatoryProperty(mandatoryProperty.name, mandatoryProperty.type, result.substr(result.indexOf(',') + 1));
        });
      } else {
        pushMandatoryProperty(mandatoryProperty.name, mandatoryProperty.type, mandatoryProperty.type === PROPERTY_TYPE.BOOLEAN ? formData[mandatoryProperty.name].value : formData[mandatoryProperty.name]);
      }
    });
    return mappedProperties;
  }

  onSubmit(formData) {
    const {
      name,
      type,
      provider
    } = formData;
    const {
      availableTypes,
      resource,
      selectedProvider
    } = this.state;
    const {
      changelog,
      onChangelogUpdate
    } = this.props;
    const selectedType = this.state.selectedType || {};
    const mandatoryProperties = this.mapMandatoryProperties(selectedType.mandatoryProperties || [], formData);
    ChangelogWriteService.createResource(changelog, resource.path, provider.value, name, (type || {}).value, mandatoryProperties);
    Object.keys(selectedType.mandatoryChildren || {}).forEach(childName => {
      const mandatoryChild = selectedType.mandatoryChildren[childName];

      if (availableTypes[selectedProvider].some(availableType => availableType.name === mandatoryChild.name)) {
        ChangelogWriteService.createResource(changelog, getResourcePath(resource.path, name), provider.value, childName, mandatoryChild.name, this.mapMandatoryProperties(mandatoryChild.mandatoryProperties, formData));
      }
    });
    onChangelogUpdate(changelog);
    this.close();
  }

  validateName(name) {
    const isAlreadyPresent = this.state.resource.children.map(childPath => extractParentWithChild(childPath).childName).some(childName => name === childName);

    if (isAlreadyPresent) {
      return 'Resource with this name already exists';
    }
  }

  nodeTypeSelected(typeName, selectedProvider, availableTypes) {
    const selectedType = availableTypes[selectedProvider].find(available => available.name === typeName);
    this.setState({
      selectedType: selectedType
    });

    if (selectedType && selectedType.allowedChildName !== '*') {
      this.setState({
        newNodeName: selectedType.allowedChildName
      });
      this.nameTextField.disabled = true;
    } else {
      this.setState({
        newNodeName: ''
      });
      this.nameTextField.disabled = false;
    }
  }

  render() {
    const {
      availableTypes,
      defaultTypes,
      isOpen,
      resource,
      selectedProvider,
      selectedType
    } = this.state;
    const providerOptions = (resource.providers || []).filter(provider => provider.modifiable).map(provider => ({
      value: provider.value,
      label: provider.label + ' Provider'
    }));
    const definesTypes = Object.keys(availableTypes).includes(selectedProvider);

    const typeSection = () => {
      const typeOptions = definesTypes ? availableTypes[selectedProvider].map(type => ({
        label: type.name,
        value: type.name
      })) : [];
      const typeMandatoryProperties = (selectedType || {}).mandatoryProperties || [];
      return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(Select, {
        label: "Type",
        name: "type",
        isClearable: false,
        isRequired: true,
        options: typeOptions,
        defaultValue: typeOptions.find(({
          label
        }) => label === defaultTypes[selectedProvider]),
        menuPosition: "fixed",
        noOptionsMessage: () => 'No type is allowed under selected parent path.',
        onChange: type => this.nodeTypeSelected(type.value, selectedProvider, availableTypes)
      }), typeMandatoryProperties.length > 0 && /*#__PURE__*/React.createElement(React.Fragment, null, typeMandatoryProperties.map(property => getMandatoryPropertyField(property))));
    };

    return /*#__PURE__*/React.createElement(ModalTransition, null, isOpen && /*#__PURE__*/React.createElement(ModalDialog, {
      scrollBehavior: "inside",
      components: {
        Header: props => /*#__PURE__*/React.createElement(ModalHeader, _extends({}, props, {
          style: {
            alignItems: 'baseline',
            flexDirection: 'column'
          }
        }), /*#__PURE__*/React.createElement(ModalTitle, null, "Create Resource"), /*#__PURE__*/React.createElement("p", null, "Parent resource: ", resource.path))
      },
      onClose: this.close
    }, definesTypes && !availableTypes[selectedProvider].length && /*#__PURE__*/React.createElement(SectionMessage, {
      appearance: "error"
    }, "Cannot create Resource below ", /*#__PURE__*/React.createElement("em", null, resource.path), ". No type is allowed for ", /*#__PURE__*/React.createElement("em", null, selectedProvider), " provider."), definesTypes && Object.keys((selectedType || {}).mandatoryChildren || []).filter(childName => !availableTypes[selectedProvider].some(availableType => availableType.name === selectedType.mandatoryChildren[childName].name)).map(childName => /*#__PURE__*/React.createElement(SectionMessage, {
      key: childName,
      title: "Selected NodeType has mandatory child Node without predefined NodeType",
      appearance: "warning"
    }, /*#__PURE__*/React.createElement("p", null, "Please make sure to create child Node ", /*#__PURE__*/React.createElement("code", null, childName), " with the type selected from allowed type list: ", /*#__PURE__*/React.createElement("i", null, selectedType.mandatoryChildren[childName].name), "."))), /*#__PURE__*/React.createElement(Form, {
      onSubmit: this.onSubmit
    }, ({
      submitted
    }) => /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(TextField, {
      autocomplete: "off",
      label: "Name",
      name: "name",
      value: this.state.newNodeName,
      ref: element => this.nameTextField = element,
      isRequired: true,
      validate: this.validateName
    }), /*#__PURE__*/React.createElement(Select, {
      label: "Resource provider",
      name: "provider",
      isRequired: true,
      value: providerOptions.find(option => option.value === selectedProvider),
      defaultValue: providerOptions[0],
      options: providerOptions,
      menuPosition: "fixed",
      noOptionEmptyMessage: () => 'None of selected providers is capable of creating node under selected parent path.',
      onChange: provider => this.setState({
        selectedProvider: provider.value
      })
    }), definesTypes && typeSection(), /*#__PURE__*/React.createElement(FormFooter, null, /*#__PURE__*/React.createElement(ButtonGroup, null, /*#__PURE__*/React.createElement(Button, {
      appearance: "primary",
      type: "submit",
      isDisabled: !selectedProvider || !selectedType && definesTypes,
      isLoading: submitted
    }, "Create"), /*#__PURE__*/React.createElement(Button, {
      appearance: "subtle",
      onClick: this.close,
      isDisabled: submitted
    }, "Cancel")))))));
  }

}