function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import React from "/apps/websight-atlaskit-esm/web-resources/react.js";
import { HashRouter, Redirect, Route, Switch } from "/apps/websight-atlaskit-esm/web-resources/react-router-dom.js";
import { ContainerHeader, HeaderSection, LayoutManager, NavigationProvider } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/navigation-next.js";
import { AvatarIcon } from "/apps/websight-admin/web-resources/Icons.js";
import { PageContentContainer } from "/apps/websight-admin/web-resources/Containers.js";
import GlobalNavigation from "/apps/websight-admin/web-resources/GlobalNavigation.js";
import Footer from "/apps/websight-admin/web-resources/Footer.js";
import { getWebFragments } from "/apps/websight-fragments-esm/web-resources/WebFragments.js";
import { errorNotification } from '/apps/websight-rest-atlaskit-client/web-resources/js/Notification.js';
import UsersPage from "./pages/UsersPage.js";
import UserDetailsPage from "./pages/UserDetailsPage.js";
import GroupsPage from "./pages/GroupsPage.js";
import GroupDetailsPage from "./pages/GroupDetailsPage.js";
import SystemUsersPage from "./pages/SystemUsersPage.js";
import SystemUserDetailsPage from "./pages/SystemUserDetailsPage.js";
import NavigationMenu from "./components/NavigationMenu.js";
import { getAuthorizableDisplayName } from "./utils/AuthorizableUtil.js";
import RecentAuthorizablesService from "./services/RecentAuthorizablesService.js";

const NavigationHeader = () => /*#__PURE__*/React.createElement(HeaderSection, null, ({
  css
}) => /*#__PURE__*/React.createElement("div", {
  style: { ...css,
    paddingBottom: 20
  }
}, /*#__PURE__*/React.createElement(ContainerHeader, {
  before: () => /*#__PURE__*/React.createElement(AvatarIcon, {
    className: "material-icons"
  }, "person_outline"),
  href: window.location.pathname,
  text: "User Manager"
})));

export default class UserManager extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      recentAuthorizables: RecentAuthorizablesService.loadRecentAuthorizablesFromLocalStorage(),
      extraActions: null
    };
    getWebFragments('websight.admin.usermanager.extra.actions', fragments => this.setState({
      extraActions: fragments
    }), errorNotification);
    this.onAuthorizableAction = this.onAuthorizableAction.bind(this);
  }

  onAuthorizableAction(actionType, authorizable = {}) {
    const {
      id,
      type,
      enabled
    } = authorizable;

    if (actionType === 'out-of-date-view') {
      this.setState(prevState => {
        const recentAuthorizables = RecentAuthorizablesService.removeFromRecent(id, this.adjustType(type), prevState.recentAuthorizables);
        return {
          recentAuthorizables
        };
      });
    } else {
      this.setState(prevState => {
        const recentAuthorizables = RecentAuthorizablesService.updateRecent({
          id,
          displayName: getAuthorizableDisplayName(authorizable),
          time: Date.now(),
          disabled: enabled === false
        }, this.adjustType(type), actionType, prevState.recentAuthorizables);
        return {
          recentAuthorizables
        };
      });
    }
  }

  adjustType(type) {
    if (type === 'system_user') {
      return 'system-user';
    }

    return type;
  }

  render() {
    const {
      recentAuthorizables,
      extraActions
    } = this.state;
    return /*#__PURE__*/React.createElement(HashRouter, null, /*#__PURE__*/React.createElement(NavigationProvider, null, /*#__PURE__*/React.createElement(LayoutManager, {
      globalNavigation: GlobalNavigation,
      productNavigation: () => null,
      containerNavigation: () => /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(NavigationHeader, null), /*#__PURE__*/React.createElement(NavigationMenu, {
        key: "leftNavigation",
        recentAuthorizables: recentAuthorizables
      })),
      experimental_horizontalGlobalNav: true
    }, /*#__PURE__*/React.createElement(PageContentContainer, null, /*#__PURE__*/React.createElement(Switch, null, /*#__PURE__*/React.createElement(Route, {
      exact: true,
      path: "/",
      render: () => /*#__PURE__*/React.createElement(Redirect, {
        to: "/users"
      })
    }), /*#__PURE__*/React.createElement(Route, {
      path: ['/users/:userId/:tabName', '/users/:userId'],
      render: routeProps => /*#__PURE__*/React.createElement(UserDetailsPage, _extends({}, routeProps, {
        onAuthorizableAction: this.onAuthorizableAction
      }))
    }), /*#__PURE__*/React.createElement(Route, {
      path: '/users',
      render: routeProps => /*#__PURE__*/React.createElement(UsersPage, _extends({}, routeProps, {
        onAuthorizableAction: this.onAuthorizableAction,
        extraActions: extraActions
      }))
    }), /*#__PURE__*/React.createElement(Route, {
      path: ['/groups/:groupId/:tabName', '/groups/:groupId'],
      render: routeProps => /*#__PURE__*/React.createElement(GroupDetailsPage, _extends({}, routeProps, {
        onAuthorizableAction: this.onAuthorizableAction
      }))
    }), /*#__PURE__*/React.createElement(Route, {
      path: '/groups',
      render: routeProps => /*#__PURE__*/React.createElement(GroupsPage, _extends({}, routeProps, {
        onAuthorizableAction: this.onAuthorizableAction,
        extraActions: extraActions
      }))
    }), /*#__PURE__*/React.createElement(Route, {
      path: ['/system-users/:userId/:tabName', '/system-users/:userId'],
      render: routeProps => /*#__PURE__*/React.createElement(SystemUserDetailsPage, _extends({}, routeProps, {
        onAuthorizableAction: this.onAuthorizableAction
      }))
    }), /*#__PURE__*/React.createElement(Route, {
      path: '/system-users',
      render: routeProps => /*#__PURE__*/React.createElement(SystemUsersPage, _extends({}, routeProps, {
        onAuthorizableAction: this.onAuthorizableAction,
        extraActions: extraActions
      }))
    }))), /*#__PURE__*/React.createElement(Footer, null))));
  }

}