import React from "/apps/websight-atlaskit-esm/web-resources/react.js";
import DropdownMenu, { DropdownItem } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/dropdown-menu.js";
import { Cell, Row } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/table-tree.js";
import Tooltip from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/tooltip.js";
import { TableRowActionButton } from "/apps/websight-admin/web-resources/Buttons.js";
import { TableRowActionButtonsContainer } from "/apps/websight-admin/web-resources/Containers.js";
import ConfirmationModal from "/apps/websight-admin/web-resources/ConfirmationModal.js";
import AuthorizablesTags from "./AuthorizablesTags.js";
import AuthorizableName from "./AuthorizableName.js";
import UserFormModal from "./UserFormModal.js";
import UserService from "../services/UserService.js";
import { fetchApplicableActions } from "/apps/websight-admin/web-resources/utils/ExtraActionsUtil.js";
export default class UserRow extends React.Component {
  constructor(props) {
    super(props);
    this.deleteUser = this.deleteUser.bind(this);
    this.state = {
      extraActions: props.extraActions
    };
  }

  toggleUserEnabled() {
    const {
      user,
      onAuthorizableAction
    } = this.props;
    UserService.toggleUserStatus(user.id, data => {
      this.props.onUpdate();

      if (onAuthorizableAction) {
        onAuthorizableAction(data.entity.enabled ? 'enable' : 'disable', data.entity);
      }
    });
  }

  deleteUser() {
    const {
      user
    } = this.props;
    UserService.deleteUser(user.id, () => {
      if (this.props.onAuthorizableAction) {
        this.props.onAuthorizableAction('delete', user);
      }

      this.props.onUpdate();
    }, () => this.deleteConfirmationModal.close());
  }

  editButton() {
    const {
      user
    } = this.props;
    return /*#__PURE__*/React.createElement(TableRowActionButton, {
      tooltipContent: "Edit",
      iconClassName: "material-icons",
      iconName: "edit",
      onClick: () => {
        this.userEditModal.open();

        if (this.props.onAuthorizableAction) {
          this.props.onAuthorizableAction('read', user);
        }
      }
    });
  }

  actionButtons() {
    const {
      user
    } = this.props;
    const {
      extraActions
    } = this.state;
    return /*#__PURE__*/React.createElement(TableRowActionButtonsContainer, null, this.editButton(), (!user.protectedUser || this.props.extraActions && this.props.extraActions.length > 0) && /*#__PURE__*/React.createElement(DropdownMenu, {
      trigger: "",
      triggerType: "button",
      onOpenChange: event => {
        if (event.isOpen) {
          fetchApplicableActions(this.props.extraActions, this.props.user.path, actions => {
            this.setState({
              extraActions: actions
            });
          });
        }
      }
    }, extraActions && extraActions.map((action, index) => {
      return /*#__PURE__*/React.createElement(DropdownItem, {
        key: index,
        onClick: () => action.onClick(user.path)
      }, action.label);
    }), !user.protectedUser && /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(DropdownItem, {
      onClick: () => this.toggleUserEnabled()
    }, this.props.user.enabled ? 'Disable' : 'Enable'), /*#__PURE__*/React.createElement(DropdownItem, {
      onClick: () => this.deleteConfirmationModal.open()
    }, "Delete"))), extraActions && extraActions.map(action => {
      return action.modal(user);
    }), /*#__PURE__*/React.createElement(ConfirmationModal, {
      buttonText: 'Delete',
      heading: 'Delete user',
      appearance: "danger",
      message: /*#__PURE__*/React.createElement("span", null, "Are you sure you want to permanently delete ", /*#__PURE__*/React.createElement("b", null, this.props.user.displayName), "?"),
      onConfirm: this.deleteUser,
      ref: element => this.deleteConfirmationModal = element
    }));
  }

  render() {
    const {
      user,
      onUpdate
    } = this.props;

    const userRowCells = () => [/*#__PURE__*/React.createElement(Cell, {
      key: `${user.id}-name`
    }, /*#__PURE__*/React.createElement(AuthorizableName, {
      authorizable: user,
      icon: "person",
      enabledAware: true
    })), /*#__PURE__*/React.createElement(Cell, {
      singleLine: true,
      key: `${user.id}-id`
    }, user.enabled ? user.id : /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("del", null, user.id), /*#__PURE__*/React.createElement("div", null, "(Disabled)"))), /*#__PURE__*/React.createElement(Cell, {
      singleLine: true,
      key: `${user.id}-statistics`
    }, /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("strong", null, "Count:"), " ", user.loginCount || '-'), /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("strong", null, "Last: "), user.lastLoggedInRelative && /*#__PURE__*/React.createElement(Tooltip, {
      content: user.lastLoggedIn,
      tag: "span"
    }, /*#__PURE__*/React.createElement("span", null, user.lastLoggedInRelative)), !user.lastLoggedInRelative && (user.lastLoggedIn || '-')))), /*#__PURE__*/React.createElement(Cell, {
      key: `${user.id}-groups`
    }, /*#__PURE__*/React.createElement(AuthorizablesTags, {
      authorizables: this.props.user.groups,
      expandable: true
    })), /*#__PURE__*/React.createElement(Cell, {
      singleLine: true,
      key: `${user.id}-actions`
    }, this.actionButtons()), /*#__PURE__*/React.createElement(UserFormModal, {
      key: `${user.id}-modal`,
      ref: element => this.userEditModal = element,
      onSaveSuccess: data => {
        onUpdate();

        if (this.props.onAuthorizableAction) {
          this.props.onAuthorizableAction('edit', data);
        }
      },
      create: false,
      user: user
    })];

    return /*#__PURE__*/React.createElement(Row, {
      key: user.id,
      itemId: user.id
    }, userRowCells());
  }

}