import React from "/apps/websight-atlaskit-esm/web-resources/react.js";
import Button, { ButtonGroup } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/button.js";
import { Checkbox } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/checkbox.js";
import { ErrorMessage, Fieldset } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/form.js";
import ModalDialog, { ModalTransition } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/modal-dialog.js";
import Select from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/select.js";
import styled from "/apps/websight-atlaskit-esm/web-resources/styled-components.js";
import { colors } from "/apps/websight-admin/web-resources/theme.js";
import PathAutosuggestion from "/apps/websight-autosuggestion-esm/web-resources/PathAutosuggestion.js";
import Form, { FormFooter } from "/apps/websight-rest-atlaskit-client/web-resources/js/Form.js";
import PermissionsService from "../../services/PermissionsService.js";
import RestrictionsEdit from "./RestrictionsEdit.js";
const PathAutosuggestionContainer = styled.div`
    flex: 1 1 auto;

    div > div > div:last-child > div > span {
        visibility: hidden
    }
`;
const emptyEntryData = {
  path: null,
  privileges: []
};

const checkboxLabelCss = () => {
  return currentStyles => {
    return { ...currentStyles,
      cursor: 'pointer'
    };
  };
};

const stringsToOptions = privileges => {
  return (privileges || []).map(privilege => ({
    label: privilege,
    value: privilege
  }));
};

const optionsToStrings = options => {
  return (options || []).map(option => option.value);
};

const FieldErrors = props => {
  const {
    errors = [],
    path
  } = props;

  if (!errors.length) {
    return null;
  }

  return /*#__PURE__*/React.createElement(React.Fragment, null, errors.map(error => error.path === path ? /*#__PURE__*/React.createElement(ErrorMessage, null, error.message) : null));
};

const arrayToObject = restrictions => {
  const restrictionObject = {};
  restrictions.map(restriction => {
    restrictionObject[restriction.type] = restriction.values;
  });
  return restrictionObject;
};

const objectToArray = restricitons => {
  const restrictionsArray = [];

  if (restricitons) {
    Object.entries(restricitons).map(([key, values]) => {
      restrictionsArray.push({
        type: key,
        values: values
      });
    });
  }

  return restrictionsArray;
};

export default class AccessControlEntryModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      availablePrivileges: [],
      availableRestrictions: [],
      defaultEntryData: emptyEntryData,
      isCreateMode: true,
      isPermissionTypeAllow: true,
      isOpen: false,
      restrictions: [],
      restrictionType: null,
      restrictionValue: null,
      validationErrors: []
    };
    this.open = this.open.bind(this);
    this.close = this.close.bind(this);
    this.updateRestrictions = this.updateRestrictions.bind(this);
    this.onFormSubmit = this.onFormSubmit.bind(this);
    this.onSubmitSuccess = this.onSubmitSuccess.bind(this);
  }

  open() {
    const {
      entryToEdit
    } = this.props;
    const isCreateMode = !entryToEdit;
    let entryData = emptyEntryData;

    if (!isCreateMode) {
      entryData = { ...entryData,
        path: entryToEdit.path,
        privileges: stringsToOptions(entryToEdit.privileges)
      };
    }

    this.setState({
      availablePrivileges: [],
      defaultEntryData: entryData,
      isCreateMode: isCreateMode,
      isOpen: true,
      isPermissionTypeAllow: isCreateMode ? true : entryToEdit.allow,
      restrictions: isCreateMode ? [] : objectToArray(entryToEdit.restrictions),
      validationErrors: []
    });
    const params = isCreateMode ? {} : {
      path: this.props.entryToEdit.path
    };
    PermissionsService.getAclEntryEditOptions(params, ({
      privileges,
      restrictions
    }) => {
      this.setState({
        availablePrivileges: privileges,
        availableRestrictions: restrictions
      });
    });
  }

  close() {
    this.setState({
      isOpen: false
    });
  }

  getCheckboxTheme(tokens, allow) {
    const setColor = (colorsObject, color) => {
      Object.keys(colorsObject).forEach(key => colorsObject[key] = color);
    };

    const setColors = (borderColor, boxColor, tickColor) => {
      setColor(tokens.icon.borderColor, borderColor);
      setColor(tokens.icon.boxColor, boxColor);
      setColor(tokens.icon.tickColor, tickColor);
    };

    if (allow) {
      setColors(colors.green, colors.green, colors.white);
    } else {
      setColors(colors.red, colors.red, colors.white);
    }

    return tokens;
  }

  onPermissionTypeChange() {
    this.setState(prevState => ({
      isPermissionTypeAllow: !prevState.isPermissionTypeAllow
    }));
  }

  updateRestrictions(restrictions) {
    this.setState({
      restrictions: restrictions
    });
  }

  onFormSubmit(requestData, onSuccess, onValidationFailure, onComplete) {
    requestData = { ...requestData,
      authorizableId: this.props.authorizableId,
      privileges: JSON.stringify(optionsToStrings(requestData.privileges)),
      restrictions: JSON.stringify(arrayToObject(this.state.restrictions))
    };
    const originalOnValidationFailure = onValidationFailure;

    onValidationFailure = data => {
      this.setState({
        validationErrors: data.entity || []
      });
      originalOnValidationFailure(data);
    };

    const originalOnComplete = onComplete;

    onComplete = data => {
      this.setState({
        validationErrors: []
      });
      originalOnComplete(data);
    };

    if (this.state.isCreateMode) {
      PermissionsService.createAclEntry(requestData, onSuccess, onValidationFailure, onComplete);
    } else {
      requestData = { ...requestData,
        entryId: this.props.entryToEdit.entryId,
        policyId: this.props.entryToEdit.policyId
      };
      PermissionsService.updateAclEntry(requestData, onSuccess, onValidationFailure, onComplete);
    }
  }

  onSubmitSuccess(data) {
    this.close();
    this.props.onSaveSuccess && this.props.onSaveSuccess(data);
  }

  render() {
    const {
      isPermissionTypeAllow,
      validationErrors,
      restrictions,
      availableRestrictions
    } = this.state;
    return /*#__PURE__*/React.createElement(ModalTransition, null, this.state.isOpen && /*#__PURE__*/React.createElement(ModalDialog, {
      heading: this.state.isCreateMode ? 'Add Access Control Entry' : 'Edit Access Control Entry',
      onClose: this.close,
      ref: elementRef => this.modalRef = elementRef
    }, /*#__PURE__*/React.createElement(Form, {
      onSubmit: this.onFormSubmit,
      onSuccess: this.onSubmitSuccess
    }, ({
      submitted
    }) => /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(PathAutosuggestionContainer, null, /*#__PURE__*/React.createElement(PathAutosuggestion, {
      label: "Path",
      name: "path",
      isRequired: true,
      parameters: {
        autosuggestionType: 'jcr-path'
      },
      noOptionsMessage: inputValue => `No resource found for "${inputValue}"`,
      noOptionEmptyMessage: "Start typing to find a resource",
      placeholder: "Choose a path (eg: /home)",
      defaultOptions: true,
      defaultValue: this.state.defaultEntryData.path || '/'
    })), /*#__PURE__*/React.createElement(Select, {
      label: "Privileges",
      name: "privileges",
      isRequired: true,
      className: "multi-select",
      classNamePrefix: "react-select",
      options: stringsToOptions(this.state.availablePrivileges),
      defaultValue: this.state.defaultEntryData.privileges,
      isMulti: true,
      placeholder: "Select...",
      menuPortalTarget: document.body,
      styles: {
        menuPortal: base => ({ ...base,
          zIndex: 9999
        })
      }
    }), /*#__PURE__*/React.createElement(Fieldset, {
      legend: "Permission Type"
    }, /*#__PURE__*/React.createElement(Checkbox, {
      name: "allow",
      label: isPermissionTypeAllow ? 'Allow' : 'Deny',
      hideLabel: true,
      defaultValue: isPermissionTypeAllow,
      isIndeterminate: !isPermissionTypeAllow,
      onChange: () => this.onPermissionTypeChange(),
      overrides: {
        Label: {
          cssFn: checkboxLabelCss()
        }
      },
      theme: (current, props) => this.getCheckboxTheme(current(props), isPermissionTypeAllow)
    })), /*#__PURE__*/React.createElement(Fieldset, {
      legend: "Restrictions"
    }, /*#__PURE__*/React.createElement(FieldErrors, {
      errors: validationErrors,
      path: "restrictions"
    }), /*#__PURE__*/React.createElement(RestrictionsEdit, {
      restrictions: restrictions,
      availableRestrictionTypes: availableRestrictions,
      onChange: this.updateRestrictions
    })), /*#__PURE__*/React.createElement(FormFooter, null, /*#__PURE__*/React.createElement(ButtonGroup, null, /*#__PURE__*/React.createElement(Button, {
      appearance: "primary",
      type: "submit",
      isLoading: submitted
    }, this.state.isCreateMode ? 'Add' : 'Save'), /*#__PURE__*/React.createElement(Button, {
      appearance: "subtle",
      onClick: this.close,
      isDisabled: submitted
    }, "Cancel")))))));
  }

}