import React from "/apps/websight-atlaskit-esm/web-resources/react.js";
import Button from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/button.js";
import Tooltip from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/tooltip.js";
import styled from "/apps/websight-atlaskit-esm/web-resources/styled-components.js";
import RestrictionRow from "./RestrictionRow.js";
const RestrictionContainer = styled.div`
    display: flex;
`;

const calculateRemainingRestrictionTypes = (restrictions, allTypes) => {
  const typesAlreadyUsed = restrictions.map(restriction => restriction.type);
  return (allTypes || []).filter(type => !typesAlreadyUsed.includes(type));
};

const emptyRestriction = {
  type: '',
  values: ['']
};
export default class RestrictionsEdit extends React.Component {
  constructor(props) {
    super(props);
    this.addRestriction = this.addRestriction.bind(this);
  }

  addRestriction() {
    const {
      restrictions,
      onChange,
      availableRestrictionTypes
    } = this.props;
    const remainingRestrictionTypes = calculateRemainingRestrictionTypes(restrictions, availableRestrictionTypes);

    if (remainingRestrictionTypes.length) {
      if (remainingRestrictionTypes.length === 1) {
        onChange([...restrictions, {
          type: remainingRestrictionTypes[0],
          values: ['']
        }]);
      } else {
        onChange([...restrictions, emptyRestriction]);
      }
    }
  }

  updateRestriction(indexToUpdate, restrictionData) {
    const {
      restrictions,
      onChange
    } = this.props;
    const updatedRestrictions = restrictions.map((restriction, index) => index === indexToUpdate ? restrictionData : restriction);
    onChange(updatedRestrictions);
  }

  removeRestriction(indexToDelete) {
    const {
      restrictions,
      onChange
    } = this.props;
    const updatedRestrictions = restrictions.filter((restriction, index) => index !== indexToDelete);
    onChange(updatedRestrictions);
  }

  render() {
    const {
      restrictions,
      availableRestrictionTypes
    } = this.props;
    const remainingRestrictionTypes = calculateRemainingRestrictionTypes(restrictions, availableRestrictionTypes);
    return /*#__PURE__*/React.createElement(React.Fragment, null, restrictions.map((restriction, index) => {
      return /*#__PURE__*/React.createElement(RestrictionRow, {
        key: index,
        restriction: restriction,
        values: restriction.values,
        onChange: newRestriction => this.updateRestriction(index, newRestriction),
        onRemove: () => this.removeRestriction(index),
        remainingRestrictionTypes: remainingRestrictionTypes
      });
    }), /*#__PURE__*/React.createElement(RestrictionContainer, null, restrictions.length < availableRestrictionTypes.length ? /*#__PURE__*/React.createElement(Button, {
      onClick: this.addRestriction
    }, "Add Restriction") : /*#__PURE__*/React.createElement(Tooltip, {
      content: "No more restrictions to add"
    }, /*#__PURE__*/React.createElement(Button, {
      isDisabled: true
    }, "Add Restriction"))));
  }

}